/*
 * Copyright 2004-2007 the Seasar Foundation and the Others.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, 
 * either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */
package org.seasar.extension.dbsession;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletRequestWrapper;
import javax.servlet.http.HttpSession;

import org.seasar.framework.container.servlet.S2ContainerServlet;
import org.seasar.framework.util.UUID;

/**
 * セッション情報をデータベースで管理するためのHttpServletRequestWrapperです。
 * 
 * @author higa
 * 
 */
public class DbHttpServletRequestWrapper extends HttpServletRequestWrapper {

    private HttpServletRequest request;

    private DbSessionStateManager sessionStateManager;

    private DbHttpSession session;

    private String requestedSessionIdFromCookie;

    private String requestedSessionIdFromURL;

    private String createdSessionId;

    /**
     * @param request
     * @param sessionStateManager
     */
    public DbHttpServletRequestWrapper(HttpServletRequest request,
            DbSessionStateManager sessionStateManager) {
        super(request);
        this.request = request;
        this.sessionStateManager = sessionStateManager;
        setupSessionId();
    }

    protected void setupSessionId() {
        requestedSessionIdFromCookie = SessionIdUtil
                .getSessionIdFromCookie(request);
        if (requestedSessionIdFromCookie == null) {
            requestedSessionIdFromURL = SessionIdUtil
                    .getSessionIdFromURL(request);
        }
        if (requestedSessionIdFromURL == null) {
            createdSessionId = UUID.create();
        }
    }

    public HttpSession getSession() {
        return getSession(true);
    }

    public HttpSession getSession(boolean create) {
        if (session != null) {
            return session;
        }
        if (!create) {
            return null;
        }
        boolean isNew = false;
        String sessionId = getRequestedSessionId();
        if (sessionId == null) {
            sessionId = createdSessionId;
            isNew = true;
        }
        session = new DbHttpSession(sessionId, sessionStateManager,
                S2ContainerServlet.getInstance().getServletContext(), isNew);
        return session;
    }

    /**
     * DbHttpSessionを返します。
     * 
     * @return
     */
    public DbHttpSession getDbHttpSession() {
        return session;
    }

    public String getRequestedSessionId() {
        if (requestedSessionIdFromCookie != null) {
            return requestedSessionIdFromCookie;
        }
        return requestedSessionIdFromURL;
    }

    /**
     * 作成されたSession Idを返します。
     * 
     * @return
     */
    public String getCreatedSessionId() {
        return createdSessionId;
    }

    /**
     * Session Idを返します。
     * 
     * @return
     */
    public String getSessionId() {
        String sessionId = getRequestedSessionId();
        if (sessionId == null) {
            sessionId = createdSessionId;
        }
        return sessionId;
    }

    public boolean isRequestedSessionIdFromCookie() {
        return requestedSessionIdFromCookie != null;
    }

    public boolean isRequestedSessionIdFromUrl() {
        return isRequestedSessionIdFromURL();
    }

    public boolean isRequestedSessionIdFromURL() {
        return requestedSessionIdFromURL != null;
    }
}
